/*******************************************************************************
 *
 *  UserExperience.c - Main Application
 *
 *  Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include <stdint.h>
#include <stdio.h>
#include <math.h>
#include "msp430.h"
#include "HAL_PMM.h"
#include "HAL_UCS.h"
#include "HAL_Board.h"
#include "HAL_Buttons.h"
#include "HAL_Cma3000.h"
#include "HAL_Dogs102x6.h"
#include "HAL_Menu.h"
#include "HAL_Wheel.h"
#include "LaunchpadDef.h"
#include "Clock.h"
#include "LPM.h"
#include "MassStorage.h"
#include "Mouse.h"
#include "PMM.h"
#include "Settings.h"
#include "PowerTest.h"
#include "DemoApps.h"
#include "SdCard.h"
#include "MenuGames.h"
#include "ff.h"

uint16_t timeoutCounter;

// This image has been created by ImageDog using 'ti_bug.jpg'
static const uint8_t tiBug[] =
{
    0x42,    // Image width in pixels
    0x08,    // Image height in rows (1 row = 8 pixels)
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
    0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0x0f, 0x0f, 0x0f, 0x0f, 0x08, 0x00, 0x00, 0x0f, 0xcf, 0xcf, 0xcf, 0xcf,
    0xcf, 0xc8, 0x00, 0x00, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f,
    0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xf8, 0xf0, 0xf0, 0xf0, 0xf0, 0xc0, 0x00, 0x07, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xfe, 0xc0, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x00, 0x70, 0x7c, 0x7e, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f,
    0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xcf, 0x0f, 0x0f, 0x0f, 0x00, 0x00, 0x01, 0x3f, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xf0, 0x00, 0x01, 0x0f, 0x0f, 0x0f, 0x0f, 0x0f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0x3f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xe0,
    0xf0, 0xfc, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x80, 0x00, 0x0e, 0xfe, 0xfe, 0xfe,
    0xfe, 0xfe, 0xfc, 0x80, 0x00, 0x0e, 0xfe, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x80, 0xe0, 0xf0, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf0, 0xe0, 0xc0, 0x80,
    0x80, 0x80, 0xc0, 0xc0, 0xf0, 0xf8, 0xfe, 0xff, 0xff, 0xff, 0xff, 0x03, 0x03, 0x01, 0x01, 0x01,
    0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x0f, 0xff, 0xff, 0xff, 0xfe, 0xfe,
    0xfc, 0xf8, 0xf0, 0xf0, 0xe0, 0xe0, 0xc0, 0xc0, 0xc0, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe0, 0xf8, 0xfe, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xf8, 0xe0, 0xc0, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
    0xc0, 0xf0, 0xf0, 0xf8, 0xfc, 0xfe, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x02, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

static const char *const mainMenu[] = {
    "=MSP-EXP430F5529=",
    "1. Clock ",
    "2. Games ",
    "3. Power Tests ",
    "4. Demo Apps ",
    "5. SD Card Access",
    "6. Settings "
};

FATFS fatfs;  

char buf0[24], buf1[24], buf2[24], buf3[24];

void InitI2C(void)
{
      P3SEL |= 0x03;                            // Assign I2C pins to USCI_B0
      P3SEL &= ~BIT2;                           //P3.3, RDY input
 
      UCB0CTL1 |= UCSWRST;                      // Enable SW reset
 
      UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
 
      UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
 
      UCB0BR0 = 250;                             // fSCL = SMCLK/250 = ~100kHz
 
      UCB0BR1 = 0;
  
      UCB0I2CSA = 0x1E;                         // Slave Address is 01Eh
 
      UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
}

void main(void)
{
//    double angle;
//    int x, y, z, i, buf[16];
    
    uint8_t contrast = *((unsigned char *)contrastSetpointAddress);
    uint8_t brightness = *((unsigned char *)brightnessSetpointAddress);

    // Initialize accelerometer offset from flash
    Cma3000_setAccel_offset(*((unsigned char *)accelXcalibrationAddress),
                            *((unsigned char *)accelYcalibrationAddress),
                            *((unsigned char *)accelZcalibrationAddress));

    // Stop WDT
    WDTCTL = WDTPW + WDTHOLD;

    // Basic GPIO initialization
    Board_init();

    // Set Vcore to accomodate for max. allowed system speed
    SetVCore(3);

    // Use 32.768kHz XTAL as reference
    LFXT_Start(XT1DRIVE_0);

    // Set system clock to max (25MHz)
    Init_FLL_Settle(25000, 762);

    SFRIFG1 = 0;
    SFRIE1 |= OFIE;

    // Globally enable interrupts
    __enable_interrupt();

    // Setup real time clock
    SetupRTC();
//    InitI2C();

    // Set up LCD
    Dogs102x6_init();
    Dogs102x6_backlightInit();

    // Contrast not programed in Flash Yet
    if (contrast == 0xFF)
        // Set Default Contrast
        contrast = 11;

    // Brightness not programed in Flash Yet
    if (brightness == 0xFF)
        // Set Default Brightness
        brightness = 11;

    Dogs102x6_setBacklight(brightness);
    Dogs102x6_setContrast(contrast);
    Dogs102x6_clearScreen();
//    for(i=0; i<16; i++) buf[i] = 0;
//    UCB0CTL1 |= UCTXSTT+UCTR;             // I2C TX, start condition
//    UCB0TXBUF = 2;
//    while((UCB0IFG & UCTXIFG) == 0);
//    UCB0TXBUF = 0;
//    while((UCB0IFG & UCTXIFG) == 0);
//    UCB0CTL1 &= ~UCTR;
//    UCB0CTL1 |= UCTXSTP;
//    while(UCB0CTL1 & UCTXSTP);
//
//    while(1)
//    {
//        while ((P3IN & BIT2) == 0);
//        //read from 0x03
//        UCB0CTL1 |= UCTXSTT+UCTR;             // I2C TX, start condition
//        UCB0TXBUF = 3;
//        while((UCB0IFG & UCTXIFG) == 0);
//        UCB0CTL1 &= ~UCTR;
//        UCB0CTL1 |= UCTXSTT;
//        for(i=0; i<6; i++)
//        {
//            while((UCB0IFG & UCRXIFG) == 0);
//            buf[i] = UCB0RXBUF;
//            UCB0IFG &= ~UCRXIFG;
//            if (i==4)
//            {
//                UCB0CTL1 |= UCTXSTP+UCTXNACK;
//            }
//        }
//        x = (buf[0]<<8) | buf[1];
//        z = (buf[2]<<8) | buf[3]; 
//        y = (buf[4]<<8) | buf[5];
//        angle= atan2((double)y,(double)x);
//        angle = angle * (180 / 3.14159265) + 180;
//        
//        sprintf(buf0, "a=%.1f", angle);
//        sprintf(buf1, "x=%d", x);
//        sprintf(buf2, "y=%d", y);
//        sprintf(buf3, "z=%d", z);
//        Dogs102x6_stringDraw(0, 0, buf1, DOGS102x6_DRAW_NORMAL);
//        Dogs102x6_stringDraw(1, 0, buf2, DOGS102x6_DRAW_NORMAL);
//        Dogs102x6_stringDraw(2, 0, buf3, DOGS102x6_DRAW_NORMAL);
//        Dogs102x6_stringDraw(3, 0, buf0, DOGS102x6_DRAW_NORMAL);
//    };
//    while(1);

    // Set up wheel
//    Wheel_init();
//    Buttons_init(BUTTON_ALL);
//    Buttons_interruptEnable(BUTTON_ALL);
//    buttonsPressed = 0;

    // Display TI logo
//    Dogs102x6_imageDraw(tiBug, 0, 16);
    
    f_mount(0, &fatfs);
    
    Dogs102x6_chineseDraw(0, 0, "ǰ¹⣬");
    Dogs102x6_chineseDraw(1, 0, "ǵ˪");
    Dogs102x6_chineseDraw(2, 0, "ͷ£");
    Dogs102x6_chineseDraw(3, 0, "ͷ˼硣");
    while(1);
    

    // Wait for button press
//    while (!buttonsPressed)
//    {
//        for (timeoutCounter = 0; timeoutCounter < 0xFFFF; timeoutCounter++)
//        {
//            if (buttonsPressed)
//                break;
//            __delay_cycles(1000);
//        }
//
//        //Timeout break
//        break;
//    }
//    buttonsPressed = 0;
//    Dogs102x6_clearScreen();

    // Main loop
//    while (1)
//    {
//        Dogs102x6_stringDraw(7, 0, "*S1=Enter S2=Esc*", DOGS102x6_DRAW_NORMAL);
//        switch (Menu_active((char **)mainMenu, 6))
//        {
//            case 1: ClockMenu(); break;
//            case 2: MenuGames(); break;
//            case 3: PowerMenu(); break;
//            case 4: DemoApp();   break;
//            case 5: SDCard();    break;
//            case 6: Settings();  break;
//            default: break;
//        }
//    }
}
